package fr.openpeople.test.fibonacci.core;

/**
 * Classe calculant la valeur de la célèbre suite de Fibonacci,
 *  selon plusieurs méthodes différentes.
 * 
 * @author KR
 */
public class SuiteFibonacci {

	/**
	 * Calcul (trivial) d'une valeur de la suite de Fibonacci.
	 * @param n paramètre entier pour lequel calculer
	 *          la valeur de la suite de Fibonacci.
	 *          Ne pas aller au-delà de 92 !
	 * @return la valeur de la suite de Fibonacci
	 *         pour le paramètre entier <code>n</code>.
	 * @throws IllegalArgumentException lorsque le paramètre
	 *         <code>n</code> est supérieur à 92.
	 */
	protected static long calculerFibonacciIter(int n)
	throws IllegalArgumentException
	{
		if (n > 92) {
			String msgErr = String.format("Paramètre trop élevé (%d) pour"
			                            + " cette méthode (max = 92).", n);
			throw new IllegalArgumentException(msgErr);
		}
		if (n <= 0) return 0;
		if (n == 1) return 1;
		long courant = 0, precedent = 1, avantPrecedent = 0;
		for (int cpt = 2; cpt <= n; cpt++) {
			courant = precedent + avantPrecedent;
			avantPrecedent = precedent;
			precedent = courant;
		}
		return courant;
	}

	/**
	 * Calcul efficace d'une valeur de la suite de Fibonacci,
	 *  par l'utilisation de la fonction de Binet.
	 * ATTENTION : pour <code>n</code> supérieur à 70,
	 *  le résultat perd en exactitude, à cause de l'imprécision
	 *  liée aux calculs sur les nombres à virgule flottante !
	 * @param n paramètre entier pour lequel calculer
	 *          la valeur de la suite de Fibonacci.
	 * @return la valeur de la suite de Fibonacci
	 *         pour le paramètre entier <code>n</code>.
	 */
	protected static double calculerFibonacciFonc(int n)
	{
		final double racine5 = Math.sqrt(5);
		final double nbOr = (1 + racine5) / 2;
		final double conjNbOr = (1 - racine5) / 2;
		if (n <= 0) return 0;
		// Formule de Binet
		return ((Math.pow(nbOr, n) - Math.pow(conjNbOr, n)) / racine5);
	}

	/**
	 * Lance le calcul d'une valeur de la suite de Fibonacci,
	 *  en employant la méthode voulue.
	 * @param n paramètre entier pour lequel calculer
	 *          la valeur de la suite de Fibonacci.
	 * @param methode méthode de calcul à employer.
	 * @return la valeur de la suite de Fibonacci
	 *         pour le paramètre entier <code>n</code>.
	 */
	public static double calculerFibonacci(
			int n,
			MethodeCalculFibonacci methode)
	{
		double resultat;
		switch (methode) {
		case CALCUL_ITERATIF:
			resultat = (double)calculerFibonacciIter(n);
			break;
		case CALCUL_FONCTIONNEL:
			resultat = calculerFibonacciFonc(n);
			break;
		default:
			throw new IllegalArgumentException("Méthode de calcul inconnue !");
		}
		return resultat;
	}

}
